@@ -31,31 +31,13 @@ template <class ValueType>
 struct StdPartitionCopyScalar {
   ValueType true_count_;
   ValueType false_count_;
-
-  // Here we implement the copy assignment operators explicitly for consistency
-  // with how the Scalar structs are implemented inside
-  // Kokkos_Parallel_Reduce.hpp.
-  KOKKOS_FUNCTION
-  void operator=(const StdPartitionCopyScalar& other) {
-    true_count_  = other.true_count_;
-    false_count_ = other.false_count_;
-  }
-
-  // this is needed for
-  // OpenMPTarget/Kokkos_OpenMPTarget_Parallel.hpp:699:21: error: no viable
-  // overloaded '=' m_returnvalue = 0;
-  //
-  KOKKOS_FUNCTION
-  void operator=(const ValueType value) {
-    true_count_  = value;
-    false_count_ = value;
-  }
 };
 
-template <class IndexType, class FirstFrom, class FirstDestTrue,
-          class FirstDestFalse, class PredType>
+template <class FirstFrom, class FirstDestTrue, class FirstDestFalse,
+          class PredType>
 struct StdPartitionCopyFunctor {
-  using value_type = StdPartitionCopyScalar<IndexType>;
+  using index_type = typename FirstFrom::difference_type;
+  using value_type = StdPartitionCopyScalar<index_type>;
 
   FirstFrom m_first_from;
   FirstDestTrue m_first_dest_true;
@@ -71,7 +53,7 @@ struct StdPartitionCopyFunctor {
         m_pred(std::move(pred)) {}
 
   KOKKOS_FUNCTION
-  void operator()(const IndexType i, value_type& update,
+  void operator()(const index_type i, value_type& update,
                   const bool final_pass) const {
     const auto& myval = m_first_from[i];
     if (final_pass) {
@@ -106,11 +88,12 @@ template <class ExecutionSpace, class InputIteratorType,
           class OutputIteratorTrueType, class OutputIteratorFalseType,
           class PredicateType>
 ::Kokkos::pair<OutputIteratorTrueType, OutputIteratorFalseType>
-partition_copy_impl(const std::string& label, const ExecutionSpace& ex,
-                    InputIteratorType from_first, InputIteratorType from_last,
-                    OutputIteratorTrueType to_first_true,
-                    OutputIteratorFalseType to_first_false,
-                    PredicateType pred) {
+partition_copy_exespace_impl(const std::string& label, const ExecutionSpace& ex,
+                             InputIteratorType from_first,
+                             InputIteratorType from_last,
+                             OutputIteratorTrueType to_first_true,
+                             OutputIteratorFalseType to_first_false,
+                             PredicateType pred) {
   // impl uses a scan, this is similar how we implemented copy_if
 
   // checks
@@ -124,12 +107,9 @@ partition_copy_impl(const std::string& label, const ExecutionSpace& ex,
     return {to_first_true, to_first_false};
   }
 
-  // aliases
-  using index_type = typename InputIteratorType::difference_type;
   using func_type =
-      StdPartitionCopyFunctor<index_type, InputIteratorType,
-                              OutputIteratorTrueType, OutputIteratorFalseType,
-                              PredicateType>;
+      StdPartitionCopyFunctor<InputIteratorType, OutputIteratorTrueType,
+                              OutputIteratorFalseType, PredicateType>;
 
   // run
   const auto num_elements =
@@ -145,6 +125,55 @@ partition_copy_impl(const std::string& label, const ExecutionSpace& ex,
           to_first_false + counts.false_count_};
 }
 
+template <class TeamHandleType, class InputIteratorType,
+          class OutputIteratorTrueType, class OutputIteratorFalseType,
+          class PredicateType>
+KOKKOS_FUNCTION ::Kokkos::pair<OutputIteratorTrueType, OutputIteratorFalseType>
+partition_copy_team_impl(const TeamHandleType& teamHandle,
+                         InputIteratorType from_first,
+                         InputIteratorType from_last,
+                         OutputIteratorTrueType to_first_true,
+                         OutputIteratorFalseType to_first_false,
+                         PredicateType pred) {
+  // impl uses a scan, this is similar how we implemented copy_if
+
+  // checks
+  Impl::static_assert_random_access_and_accessible(
+      teamHandle, from_first, to_first_true, to_first_false);
+  Impl::static_assert_iterators_have_matching_difference_type(
+      from_first, to_first_true, to_first_false);
+  Impl::expect_valid_range(from_first, from_last);
+
+  if (from_first == from_last) {
+    return {to_first_true, to_first_false};
+  }
+
+  const std::size_t num_elements =
+      Kokkos::Experimental::distance(from_first, from_last);
+
+  // FIXME: there is no parallel_scan overload that accepts TeamThreadRange and
+  // return_value, so temporarily serial implementation is used
+  using counts_t  = ::Kokkos::pair<std::size_t, std::size_t>;
+  counts_t counts = {};
+  Kokkos::single(
+      Kokkos::PerTeam(teamHandle),
+      [=](counts_t& lcounts) {
+        lcounts = {};
+        for (std::size_t i = 0; i < num_elements; ++i) {
+          const auto& myval = from_first[i];
+          if (pred(myval)) {
+            to_first_true[lcounts.first++] = myval;
+          } else {
+            to_first_false[lcounts.second++] = myval;
+          }
+        }
+      },
+      counts);
+  // no barrier needed since single above broadcasts to all members
+
+  return {to_first_true + counts.first, to_first_false + counts.second};
+}
+
 }  // namespace Impl
 }  // namespace Experimental
 }  // namespace Kokkos
