@@ -21,30 +21,21 @@
 #include "Kokkos_Constraints.hpp"
 #include "Kokkos_HelperPredicates.hpp"
 #include <std_algorithms/Kokkos_Distance.hpp>
-#include <std_algorithms/Kokkos_Swap.hpp>
 #include <string>
 
 namespace Kokkos {
 namespace Experimental {
 namespace Impl {
 
-template <class IndexType, class IteratorType1, class IteratorType2>
+template <class IteratorType1, class IteratorType2>
 struct StdSwapRangesFunctor {
+  using index_type = typename IteratorType1::difference_type;
   IteratorType1 m_first1;
   IteratorType2 m_first2;
 
   KOKKOS_FUNCTION
-  void operator()(IndexType i) const {
-    // the swap below is doing the same thing, but
-    // for Intel 18.0.5 does not work.
-    // But putting the impl directly here, it works.
-#ifdef KOKKOS_COMPILER_INTEL
-    typename IteratorType1::value_type tmp = std::move(m_first1[i]);
-    m_first1[i]                            = std::move(m_first2[i]);
-    m_first2[i]                            = std::move(tmp);
-#else
-    ::Kokkos::Experimental::swap(m_first1[i], m_first2[i]);
-#endif
+  void operator()(index_type i) const {
+    ::Kokkos::kokkos_swap(m_first1[i], m_first2[i]);
   }
 
   KOKKOS_FUNCTION
@@ -53,30 +44,48 @@ struct StdSwapRangesFunctor {
 };
 
 template <class ExecutionSpace, class IteratorType1, class IteratorType2>
-IteratorType2 swap_ranges_impl(const std::string& label,
-                               const ExecutionSpace& ex, IteratorType1 first1,
-                               IteratorType1 last1, IteratorType2 first2) {
+IteratorType2 swap_ranges_exespace_impl(const std::string& label,
+                                        const ExecutionSpace& ex,
+                                        IteratorType1 first1,
+                                        IteratorType1 last1,
+                                        IteratorType2 first2) {
   // checks
   Impl::static_assert_random_access_and_accessible(ex, first1, first2);
   Impl::static_assert_iterators_have_matching_difference_type(first1, first2);
   Impl::expect_valid_range(first1, last1);
 
-  // aliases
-  using index_type = typename IteratorType1::difference_type;
-  using func_t = StdSwapRangesFunctor<index_type, IteratorType1, IteratorType2>;
-
   // run
   const auto num_elements_to_swap =
       Kokkos::Experimental::distance(first1, last1);
   ::Kokkos::parallel_for(
       label, RangePolicy<ExecutionSpace>(ex, 0, num_elements_to_swap),
-      func_t(first1, first2));
+      StdSwapRangesFunctor(first1, first2));
   ex.fence("Kokkos::swap_ranges: fence after operation");
 
   // return
   return first2 + num_elements_to_swap;
 }
 
+template <class TeamHandleType, class IteratorType1, class IteratorType2>
+KOKKOS_FUNCTION IteratorType2
+swap_ranges_team_impl(const TeamHandleType& teamHandle, IteratorType1 first1,
+                      IteratorType1 last1, IteratorType2 first2) {
+  // checks
+  Impl::static_assert_random_access_and_accessible(teamHandle, first1, first2);
+  Impl::static_assert_iterators_have_matching_difference_type(first1, first2);
+  Impl::expect_valid_range(first1, last1);
+
+  // run
+  const auto num_elements_to_swap =
+      Kokkos::Experimental::distance(first1, last1);
+  ::Kokkos::parallel_for(TeamThreadRange(teamHandle, 0, num_elements_to_swap),
+                         StdSwapRangesFunctor(first1, first2));
+  teamHandle.team_barrier();
+
+  // return
+  return first2 + num_elements_to_swap;
+}
+
 }  // namespace Impl
 }  // namespace Experimental
 }  // namespace Kokkos
