@@ -52,18 +52,7 @@ HostThreadTeamDataSingleton::HostThreadTeamDataSingleton()
       num_pool_reduce_bytes, num_team_reduce_bytes, num_team_shared_bytes,
       num_thread_local_bytes);
 
-  void* ptr = nullptr;
-  try {
-    ptr = space.allocate(alloc_bytes);
-  } catch (Kokkos::Experimental::RawMemoryAllocationFailure const& f) {
-    // For now, just rethrow the error message with a note
-    // Note that this could, in turn, trigger an out of memory exception,
-    // but it's pretty unlikely, so we won't worry about it for now.
-    // TODO reasonable error message when `std::string` causes OOM error
-    Kokkos::Impl::throw_runtime_exception(
-        std::string("Failure to allocate scratch memory:  ") +
-        f.get_error_message());
-  }
+  void* ptr = space.allocate("Kokkos::Impl::HostThreadTeamData", alloc_bytes);
 
   HostThreadTeamData::scratch_assign(
       ptr, alloc_bytes, num_pool_reduce_bytes, num_team_reduce_bytes,
