@@ -78,12 +78,12 @@ std::string demangle(const std::string& name) {
 
 class Stacktrace {
  public:
-  Stacktrace()                  = delete;
-  Stacktrace(const Stacktrace&) = delete;
+  Stacktrace()                             = delete;
+  Stacktrace(const Stacktrace&)            = delete;
   Stacktrace& operator=(const Stacktrace&) = delete;
   Stacktrace(Stacktrace&&)                 = delete;
-  Stacktrace& operator=(Stacktrace&&) = delete;
-  ~Stacktrace()                       = delete;
+  Stacktrace& operator=(Stacktrace&&)      = delete;
+  ~Stacktrace()                            = delete;
 
   // These are public only to avoid wasting an extra stacktrace line.
   // See save_stacktrace below.
@@ -134,7 +134,7 @@ void for_each_token(const std::string& s, Callback c) {
     const size_t end   = find_first_whitespace(s, cur);
     const bool last    = (end == std::string::npos);
     const size_t count = last ? end : size_t(end - cur);
-    c(s.substr(cur, count), last);
+    c(s.substr(cur, count));
     cur = find_first_non_whitespace(s, end);
   }
 }
@@ -147,7 +147,6 @@ void for_each_token(const std::string& s, Callback c) {
 struct main_column_info {
   bool found_main;
   size_t main_col;
-  std::vector<size_t> main_col_lens;
 };
 
 main_column_info find_main_column(const std::vector<std::string>& traceback) {
@@ -155,7 +154,7 @@ main_column_info find_main_column(const std::vector<std::string>& traceback) {
   size_t main_col = 0;
   for (auto&& entry : traceback) {
     size_t col_count = 0;
-    for_each_token(entry, [&](const std::string& s, bool) {
+    for_each_token(entry, [&](const std::string& s) {
       const size_t pos = s.find("main");
       if (pos != std::string::npos) {
         found_main = true;
@@ -168,52 +167,33 @@ main_column_info find_main_column(const std::vector<std::string>& traceback) {
     }
   }
 
-  // Make another pass to get the column lengths.
-  // Only demangle the column of functions.
-  std::vector<size_t> max_col_lengths;
-  for (auto&& entry : traceback) {
-    size_t col_count = 0;
-    for_each_token(entry, [&](const std::string& s, bool) {
-      const size_t cur_col_len =
-          (found_main && col_count == main_col) ? demangle(s).size() : s.size();
-      ++col_count;
-      if (max_col_lengths.size() < col_count) {
-        max_col_lengths.push_back(cur_col_len);
-      } else {
-        const size_t old_max_len = max_col_lengths[col_count - 1];
-        if (old_max_len < cur_col_len) {
-          max_col_lengths[col_count - 1] = cur_col_len;
-        }
-      }
-    });
-  }
-  return main_column_info{found_main, main_col, max_col_lengths};
+  return main_column_info{found_main, main_col};
 }
 
-void demangle_and_print_traceback_entry(
-    std::ostream& out, const std::string& traceback_entry,
-    const bool found_main, const size_t main_col,
-    const std::vector<size_t>& max_col_lens) {
+void demangle_and_print_traceback_entry(std::ostream& out,
+                                        const std::string& traceback_entry,
+                                        const bool found_main,
+                                        const size_t main_col) {
   std::vector<std::string> tokens;
   size_t cur_col = 0;
-  for_each_token(traceback_entry, [&](const std::string& s, bool last) {
-    const size_t old_width(out.width());
-    out.width(max_col_lens[cur_col]);
-    try {
-      if (found_main && cur_col == main_col) {
-        out << demangle(s);
-      } else {
-        out << s;
-      }
-      if (!last) {
-        out << " ";
-      }
-      ++cur_col;
-    } catch (...) {
-      out.width(old_width);
-      throw;
+
+  // Print the address column first
+  for_each_token(traceback_entry, [&](const std::string& s) {
+    if (!(found_main && cur_col == main_col)) {
+      out << s;
     }
-    out.width(old_width);
+    ++cur_col;
+  });
+
+  out << " ";
+
+  // Then the function name
+  cur_col = 0;
+  for_each_token(traceback_entry, [&](const std::string& s) {
+    if (found_main && cur_col == main_col) {
+      out << demangle(s);
+    }
+    ++cur_col;
   });
 }
 
@@ -222,7 +202,7 @@ void demangle_and_print_traceback(std::ostream& out,
   const auto result = find_main_column(traceback);
   for (auto&& entry : traceback) {
     demangle_and_print_traceback_entry(out, entry, result.found_main,
-                                       result.main_col, result.main_col_lens);
+                                       result.main_col);
     out << std::endl;
   }
 }
