@@ -18,6 +18,7 @@
 #define KOKKOS_IMPL_KOKKOS_TOOLS_GENERIC_HPP
 
 #include <impl/Kokkos_Profiling.hpp>
+#include <impl/Kokkos_FunctorAnalysis.hpp>
 
 #include <Kokkos_Core_fwd.hpp>
 #include <Kokkos_ExecPolicy.hpp>
@@ -35,6 +36,10 @@ namespace Impl {
 static std::map<std::string, Kokkos::Tools::Experimental::TeamSizeTuner>
     team_tuners;
 
+static std::map<std::string,
+                Kokkos::Tools::Experimental::RangePolicyOccupancyTuner>
+    range_policy_tuners;
+
 template <int Rank>
 using MDRangeTuningMap =
     std::map<std::string, Kokkos::Tools::Experimental::MDRangeTuner<Rank>>;
@@ -44,13 +49,17 @@ static MDRangeTuningMap<Rank> mdrange_tuners;
 
 // For any policies without a tuning implementation, with a reducer
 template <class ReducerType, class ExecPolicy, class Functor, typename TagType>
-void tune_policy(const size_t, const std::string&, ExecPolicy&, const Functor&,
-                 TagType) {}
+auto tune_policy(const size_t, const std::string&, const ExecPolicy& policy,
+                 const Functor&, TagType) {
+  return policy;
+}
 
 // For any policies without a tuning implementation, without a reducer
 template <class ExecPolicy, class Functor, typename TagType>
-void tune_policy(const size_t, const std::string&, ExecPolicy&, const Functor&,
-                 const TagType&) {}
+auto tune_policy(const size_t, const std::string&, const ExecPolicy& policy,
+                 const Functor&, const TagType&) {
+  return policy;
+}
 
 /**
  * Tuning for parallel_fors and parallel_scans is a fairly simple process.
@@ -80,6 +89,14 @@ struct SimpleTeamSizeCalculator {
     auto max = policy.team_size_max(functor, tag);
     return max;
   }
+  template <typename Policy, typename FunctorReducer>
+  int get_max_team_size(const Policy& policy,
+                        const FunctorReducer& functor_reducer,
+                        const Kokkos::ParallelReduceTag tag) {
+    auto max = policy.team_size_max(functor_reducer.get_functor(),
+                                    functor_reducer.get_reducer(), tag);
+    return max;
+  }
   template <typename Policy, typename Functor, typename Tag>
   int get_recommended_team_size(const Policy& policy, const Functor& functor,
                                 const Tag tag) {
@@ -94,15 +111,14 @@ struct SimpleTeamSizeCalculator {
     using driver     = Kokkos::Impl::ParallelFor<Functor, Policy, exec_space>;
     return driver::max_tile_size_product(policy, functor);
   }
-  template <typename Policy, typename Functor>
+  template <typename Policy, typename FunctorReducer>
   int get_mdrange_max_tile_size_product(const Policy& policy,
-                                        const Functor& functor,
+                                        const FunctorReducer& functor_reducer,
                                         const Kokkos::ParallelReduceTag&) {
     using exec_space = typename Policy::execution_space;
     using driver =
-        Kokkos::Impl::ParallelReduce<Functor, Policy, Kokkos::InvalidType,
-                                     exec_space>;
-    return driver::max_tile_size_product(policy, functor);
+        Kokkos::Impl::ParallelReduce<FunctorReducer, Policy, exec_space>;
+    return driver::max_tile_size_product(policy, functor_reducer.get_functor());
   }
 };
 
@@ -112,41 +128,43 @@ struct SimpleTeamSizeCalculator {
 // constructible from a reference to an
 // instance of their value_type so we construct
 // a value_type and temporary reducer here
-template <typename ReducerType>
 struct ComplexReducerSizeCalculator {
-  template <typename Policy, typename Functor, typename Tag>
-  int get_max_team_size(const Policy& policy, const Functor& functor,
-                        const Tag tag) {
-    using value_type = typename ReducerType::value_type;
-    value_type value;
-    ReducerType reducer_example = ReducerType(value);
-    return policy.team_size_max(functor, reducer_example, tag);
+  template <typename Policy, typename FunctorReducer, typename Tag>
+  int get_max_team_size(const Policy& policy,
+                        const FunctorReducer& functor_reducer, const Tag tag) {
+    return policy.team_size_max(functor_reducer.get_functor(),
+                                functor_reducer.get_reducer(), tag);
   }
-  template <typename Policy, typename Functor, typename Tag>
-  int get_recommended_team_size(const Policy& policy, const Functor& functor,
+  template <typename Policy, typename FunctorReducer, typename Tag>
+  int get_recommended_team_size(const Policy& policy,
+                                const FunctorReducer& functor_reducer,
                                 const Tag tag) {
-    using value_type = typename ReducerType::value_type;
-    value_type value;
-    ReducerType reducer_example = ReducerType(value);
-    return policy.team_size_recommended(functor, reducer_example, tag);
+    return policy.team_size_recommended(functor_reducer.get_functor(),
+                                        functor_reducer.get_reducer(), tag);
   }
-  template <typename Policy, typename Functor>
+  template <typename Policy, typename FunctorReducer>
   int get_mdrange_max_tile_size_product(const Policy& policy,
-                                        const Functor& functor,
+                                        const FunctorReducer& functor_reducer,
                                         const Kokkos::ParallelReduceTag&) {
     using exec_space = typename Policy::execution_space;
     using driver =
-        Kokkos::Impl::ParallelReduce<Functor, Policy, ReducerType, exec_space>;
-    return driver::max_tile_size_product(policy, functor);
+        Kokkos::Impl::ParallelReduce<FunctorReducer, Policy, exec_space>;
+    return driver::max_tile_size_product(policy, functor_reducer.get_functor());
   }
 };
 
+template <typename Policy>
+auto default_tuned_version_of(const Policy& policy) {
+  return policy;
+}
+
 }  // namespace Impl
 
 template <class Tuner, class Functor, class TagType,
           class TuningPermissionFunctor, class Map, class Policy>
-void generic_tune_policy(const std::string& label_in, Map& map, Policy& policy,
-                         const Functor& functor, const TagType& tag,
+auto generic_tune_policy(const std::string& label_in, Map& map,
+                         const Policy& policy, const Functor& functor,
+                         const TagType& tag,
                          const TuningPermissionFunctor& should_tune) {
   if (should_tune(policy)) {
     std::string label = label_in;
@@ -165,13 +183,15 @@ void generic_tune_policy(const std::string& label_in, Map& map, Policy& policy,
       }
       return my_tuner;
     }();
-    tuner_iter->second.tune(policy);
+    return tuner_iter->second.tune(policy);
   }
+  return Impl::default_tuned_version_of(policy);
 }
 template <class Tuner, class ReducerType, class Functor, class TagType,
           class TuningPermissionFunctor, class Map, class Policy>
-void generic_tune_policy(const std::string& label_in, Map& map, Policy& policy,
-                         const Functor& functor, const TagType& tag,
+auto generic_tune_policy(const std::string& label_in, Map& map,
+                         const Policy& policy, const Functor& functor,
+                         const TagType& tag,
                          const TuningPermissionFunctor& should_tune) {
   if (should_tune(policy)) {
     std::string label = label_in;
@@ -184,24 +204,23 @@ void generic_tune_policy(const std::string& label_in, Map& map, Policy& policy,
     auto tuner_iter = [&]() {
       auto my_tuner = map.find(label);
       if (my_tuner == map.end()) {
-        return (map.emplace(
-                       label,
-                       Tuner(label, policy, functor, tag,
-                             Impl::ComplexReducerSizeCalculator<ReducerType>{}))
+        return (map.emplace(label, Tuner(label, policy, functor, tag,
+                                         Impl::ComplexReducerSizeCalculator{}))
                     .first);
       }
       return my_tuner;
     }();
-    tuner_iter->second.tune(policy);
+    return tuner_iter->second.tune(policy);
   }
+  return Impl::default_tuned_version_of(policy);
 }
 
 // tune a TeamPolicy, without reducer
 template <class Functor, class TagType, class... Properties>
-void tune_policy(const size_t /**tuning_context*/, const std::string& label_in,
-                 Kokkos::TeamPolicy<Properties...>& policy,
+auto tune_policy(const size_t /**tuning_context*/, const std::string& label_in,
+                 const Kokkos::TeamPolicy<Properties...>& policy,
                  const Functor& functor, const TagType& tag) {
-  generic_tune_policy<Experimental::TeamSizeTuner>(
+  return generic_tune_policy<Experimental::TeamSizeTuner>(
       label_in, team_tuners, policy, functor, tag,
       [](const Kokkos::TeamPolicy<Properties...>& candidate_policy) {
         return (candidate_policy.impl_auto_team_size() ||
@@ -211,10 +230,10 @@ void tune_policy(const size_t /**tuning_context*/, const std::string& label_in,
 
 // tune a TeamPolicy, with reducer
 template <class ReducerType, class Functor, class TagType, class... Properties>
-void tune_policy(const size_t /**tuning_context*/, const std::string& label_in,
-                 Kokkos::TeamPolicy<Properties...>& policy,
+auto tune_policy(const size_t /**tuning_context*/, const std::string& label_in,
+                 const Kokkos::TeamPolicy<Properties...>& policy,
                  const Functor& functor, const TagType& tag) {
-  generic_tune_policy<Experimental::TeamSizeTuner, ReducerType>(
+  return generic_tune_policy<Experimental::TeamSizeTuner, ReducerType>(
       label_in, team_tuners, policy, functor, tag,
       [](const Kokkos::TeamPolicy<Properties...>& candidate_policy) {
         return (candidate_policy.impl_auto_team_size() ||
@@ -222,14 +241,97 @@ void tune_policy(const size_t /**tuning_context*/, const std::string& label_in,
       });
 }
 
+template <class Functor, class TagType, class... Properties>
+auto tune_occupancy_controlled_policy(
+    const size_t /**tuning_context*/, const std::string& label_in,
+    const Kokkos::RangePolicy<Properties...>& policy, const Functor& functor,
+    const TagType& tag) {
+  return generic_tune_policy<Experimental::RangePolicyOccupancyTuner>(
+      label_in, range_policy_tuners, policy, functor, tag,
+      [](const Kokkos::RangePolicy<Properties...>& candidate_policy) {
+        return candidate_policy.impl_get_occupancy_control().should_tune();
+      });
+}
+template <class Functor, class TagType, class... Properties>
+auto tune_range_policy(const size_t tuning_context, const std::string& label_in,
+                       const Kokkos::RangePolicy<Properties...>& policy,
+                       const Functor& functor, const TagType& tag,
+                       std::true_type) {
+  return tune_occupancy_controlled_policy(tuning_context, label_in, policy,
+                                          functor, tag);
+}
+template <class Functor, class TagType, class... Properties>
+auto tune_range_policy(const size_t /**tuning_context*/,
+                       const std::string& /*label_in*/,
+                       const Kokkos::RangePolicy<Properties...>& policy,
+                       const Functor& /**functor*/, const TagType& /**tag*/,
+                       std::false_type) {
+  return policy;
+}
+
+// Reducer versions
+template <class RT, class Functor, class TagType, class... Properties>
+auto tune_occupancy_controlled_policy(
+    const size_t /**tuning_context*/, const std::string& label_in,
+    const Kokkos::RangePolicy<Properties...>& policy, const Functor& functor,
+    const TagType& tag) {
+  return generic_tune_policy<Experimental::RangePolicyOccupancyTuner>(
+      label_in, range_policy_tuners, policy, functor, tag,
+      [](const Kokkos::RangePolicy<Properties...>& candidate_policy) {
+        return candidate_policy.impl_get_occupancy_control().should_tune();
+      });
+}
+template <class RT, class Functor, class TagType, class... Properties>
+auto tune_range_policy(const size_t tuning_context, const std::string& label_in,
+                       const Kokkos::RangePolicy<Properties...>& policy,
+                       const Functor& functor, const TagType& tag,
+                       std::true_type) {
+  return tune_occupancy_controlled_policy<RT>(tuning_context, label_in, policy,
+                                              functor, tag);
+}
+template <class ReducerType, class Functor, class TagType, class... Properties>
+auto tune_range_policy(const size_t /**tuning_context*/,
+                       const std::string& /**label_in*/,
+                       const Kokkos::RangePolicy<Properties...>& policy,
+                       const Functor& /**functor*/, const TagType& /**tag*/,
+                       std::false_type) {
+  return policy;
+}
+
+// tune a RangePolicy, without reducer
+template <class Functor, class TagType, class... Properties>
+auto tune_policy(const size_t tuning_context, const std::string& label_in,
+                 const Kokkos::RangePolicy<Properties...>& policy,
+                 const Functor& functor, const TagType& tag) {
+  using policy_t = Kokkos::RangePolicy<Properties...>;
+  using has_desired_occupancy =
+      typename std::is_same<typename policy_t::occupancy_control,
+                            Kokkos::Experimental::DesiredOccupancy>::type;
+  return tune_range_policy(tuning_context, label_in, policy, functor, tag,
+                           has_desired_occupancy{});
+}
+
+// tune a RangePolicy, with reducer
+template <class ReducerType, class Functor, class TagType, class... Properties>
+auto tune_policy(const size_t tuning_context, const std::string& label_in,
+                 const Kokkos::RangePolicy<Properties...>& policy,
+                 const Functor& functor, const TagType& tag) {
+  using policy_t = Kokkos::RangePolicy<Properties...>;
+  using has_desired_occupancy =
+      typename std::is_same<typename policy_t::occupancy_control,
+                            Kokkos::Experimental::DesiredOccupancy>::type;
+  return tune_range_policy<ReducerType>(tuning_context, label_in, policy,
+                                        functor, tag, has_desired_occupancy{});
+}
+
 // tune a MDRangePolicy, without reducer
 template <class Functor, class TagType, class... Properties>
-void tune_policy(const size_t /**tuning_context*/, const std::string& label_in,
-                 Kokkos::MDRangePolicy<Properties...>& policy,
+auto tune_policy(const size_t /**tuning_context*/, const std::string& label_in,
+                 const Kokkos::MDRangePolicy<Properties...>& policy,
                  const Functor& functor, const TagType& tag) {
   using Policy              = Kokkos::MDRangePolicy<Properties...>;
   static constexpr int rank = Policy::rank;
-  generic_tune_policy<Experimental::MDRangeTuner<rank>>(
+  return generic_tune_policy<Experimental::MDRangeTuner<rank>>(
       label_in, mdrange_tuners<rank>, policy, functor, tag,
       [](const Policy& candidate_policy) {
         return candidate_policy.impl_tune_tile_size();
@@ -238,12 +340,12 @@ void tune_policy(const size_t /**tuning_context*/, const std::string& label_in,
 
 // tune a MDRangePolicy, with reducer
 template <class ReducerType, class Functor, class TagType, class... Properties>
-void tune_policy(const size_t /**tuning_context*/, const std::string& label_in,
-                 Kokkos::MDRangePolicy<Properties...>& policy,
+auto tune_policy(const size_t /**tuning_context*/, const std::string& label_in,
+                 const Kokkos::MDRangePolicy<Properties...>& policy,
                  const Functor& functor, const TagType& tag) {
   using Policy              = Kokkos::MDRangePolicy<Properties...>;
   static constexpr int rank = Policy::rank;
-  generic_tune_policy<Experimental::MDRangeTuner<rank>, ReducerType>(
+  return generic_tune_policy<Experimental::MDRangeTuner<rank>, ReducerType>(
       label_in, mdrange_tuners<rank>, policy, functor, tag,
       [](const Policy& candidate_policy) {
         return candidate_policy.impl_tune_tile_size();
@@ -253,31 +355,35 @@ void tune_policy(const size_t /**tuning_context*/, const std::string& label_in,
 template <class ReducerType>
 struct ReductionSwitcher {
   template <class Functor, class TagType, class ExecPolicy>
-  static void tune(const size_t tuning_context, const std::string& label,
-                   ExecPolicy& policy, const Functor& functor,
+  static auto tune(const size_t tuning_context, const std::string& label,
+                   const ExecPolicy& policy, const Functor& functor,
                    const TagType& tag) {
     if (Kokkos::tune_internals()) {
-      tune_policy<ReducerType>(tuning_context, label, policy, functor, tag);
+      return tune_policy<ReducerType>(tuning_context, label, policy, functor,
+                                      tag);
     }
+    return Impl::default_tuned_version_of(policy);
   }
 };
 
 template <>
 struct ReductionSwitcher<Kokkos::InvalidType> {
   template <class Functor, class TagType, class ExecPolicy>
-  static void tune(const size_t tuning_context, const std::string& label,
-                   ExecPolicy& policy, const Functor& functor,
+  static auto tune(const size_t tuning_context, const std::string& label,
+                   const ExecPolicy& policy, const Functor& functor,
                    const TagType& tag) {
     if (Kokkos::tune_internals()) {
-      tune_policy(tuning_context, label, policy, functor, tag);
+      return tune_policy(tuning_context, label, policy, functor, tag);
     }
+    return Impl::default_tuned_version_of(policy);
   }
 };
 
 template <class Tuner, class Functor, class TagType,
           class TuningPermissionFunctor, class Map, class Policy>
 void generic_report_results(const std::string& label_in, Map& map,
-                            Policy& policy, const Functor&, const TagType&,
+                            const Policy& policy, const Functor&,
+                            const TagType&,
                             const TuningPermissionFunctor& should_tune) {
   if (should_tune(policy)) {
     std::string label = label_in;
@@ -294,14 +400,14 @@ void generic_report_results(const std::string& label_in, Map& map,
 
 // report results for a policy type we don't tune (do nothing)
 template <class ExecPolicy, class Functor, typename TagType>
-void report_policy_results(const size_t, const std::string&, ExecPolicy&,
+void report_policy_results(const size_t, const std::string&, const ExecPolicy&,
                            const Functor&, const TagType&) {}
 
 // report results for a TeamPolicy
 template <class Functor, class TagType, class... Properties>
 void report_policy_results(const size_t /**tuning_context*/,
                            const std::string& label_in,
-                           Kokkos::TeamPolicy<Properties...>& policy,
+                           const Kokkos::TeamPolicy<Properties...>& policy,
                            const Functor& functor, const TagType& tag) {
   generic_report_results<Experimental::TeamSizeTuner>(
       label_in, team_tuners, policy, functor, tag,
@@ -315,7 +421,7 @@ void report_policy_results(const size_t /**tuning_context*/,
 template <class Functor, class TagType, class... Properties>
 void report_policy_results(const size_t /**tuning_context*/,
                            const std::string& label_in,
-                           Kokkos::MDRangePolicy<Properties...>& policy,
+                           const Kokkos::MDRangePolicy<Properties...>& policy,
                            const Functor& functor, const TagType& tag) {
   using Policy              = Kokkos::MDRangePolicy<Properties...>;
   static constexpr int rank = Policy::rank;
@@ -326,6 +432,20 @@ void report_policy_results(const size_t /**tuning_context*/,
       });
 }
 
+// report results for an MDRangePolicy
+template <class Functor, class TagType, class... Properties>
+void report_policy_results(const size_t /**tuning_context*/,
+                           const std::string& label_in,
+                           const Kokkos::RangePolicy<Properties...>& policy,
+                           const Functor& functor, const TagType& tag) {
+  using Policy = Kokkos::RangePolicy<Properties...>;
+  generic_report_results<Experimental::RangePolicyOccupancyTuner>(
+      label_in, range_policy_tuners, policy, functor, tag, [](const Policy&) {
+        return Kokkos::RangePolicy<
+            Properties...>::traits::experimental_contains_desired_occupancy;
+      });
+}
+
 }  // namespace Impl
 
 }  // namespace Experimental
@@ -333,8 +453,11 @@ void report_policy_results(const size_t /**tuning_context*/,
 namespace Impl {
 
 template <class ExecPolicy, class FunctorType>
-void begin_parallel_for(ExecPolicy& policy, FunctorType& functor,
+auto begin_parallel_for(const ExecPolicy& policy, FunctorType& functor,
                         const std::string& label, uint64_t& kpID) {
+  using response_type =
+      Kokkos::Tools::Impl::ToolResponse<ExecPolicy, FunctorType>;
+  response_type response{policy};
   if (Kokkos::Tools::profileLibraryLoaded()) {
     Kokkos::Impl::ParallelConstructName<FunctorType,
                                         typename ExecPolicy::work_tag>
@@ -344,18 +467,19 @@ void begin_parallel_for(ExecPolicy& policy, FunctorType& functor,
         &kpID);
   }
 #ifdef KOKKOS_ENABLE_TUNING
-  size_t context_id = Kokkos::Tools::Experimental::get_new_context_id();
+  size_t context_id = Kokkos::Tools::Experimental::get_current_context_id();
   if (Kokkos::tune_internals()) {
-    Experimental::Impl::tune_policy(context_id, label, policy, functor,
-                                    Kokkos::ParallelForTag{});
+    return response_type{Kokkos::Tools::Experimental::Impl::tune_policy(
+        context_id, label, policy, functor, Kokkos::ParallelForTag{})};
   }
 #else
   (void)functor;
 #endif
+  return response;
 }
 
 template <class ExecPolicy, class FunctorType>
-void end_parallel_for(ExecPolicy& policy, FunctorType& functor,
+void end_parallel_for(const ExecPolicy& policy, FunctorType& functor,
                       const std::string& label, uint64_t& kpID) {
   if (Kokkos::Tools::profileLibraryLoaded()) {
     Kokkos::Tools::endParallelFor(kpID);
@@ -374,8 +498,11 @@ void end_parallel_for(ExecPolicy& policy, FunctorType& functor,
 }
 
 template <class ExecPolicy, class FunctorType>
-void begin_parallel_scan(ExecPolicy& policy, FunctorType& functor,
+auto begin_parallel_scan(const ExecPolicy& policy, FunctorType& functor,
                          const std::string& label, uint64_t& kpID) {
+  using response_type =
+      Kokkos::Tools::Impl::ToolResponse<ExecPolicy, FunctorType>;
+  response_type response{policy};
   if (Kokkos::Tools::profileLibraryLoaded()) {
     Kokkos::Impl::ParallelConstructName<FunctorType,
                                         typename ExecPolicy::work_tag>
@@ -385,18 +512,19 @@ void begin_parallel_scan(ExecPolicy& policy, FunctorType& functor,
         &kpID);
   }
 #ifdef KOKKOS_ENABLE_TUNING
-  size_t context_id = Kokkos::Tools::Experimental::get_new_context_id();
+  size_t context_id = Kokkos::Tools::Experimental::get_current_context_id();
   if (Kokkos::tune_internals()) {
-    Experimental::Impl::tune_policy(context_id, label, policy, functor,
-                                    Kokkos::ParallelScanTag{});
+    return response_type{Kokkos::Tools::Experimental::Impl::tune_policy(
+        context_id, label, policy, functor, Kokkos::ParallelScanTag{})};
   }
 #else
   (void)functor;
 #endif
+  return response;
 }
 
 template <class ExecPolicy, class FunctorType>
-void end_parallel_scan(ExecPolicy& policy, FunctorType& functor,
+void end_parallel_scan(const ExecPolicy& policy, FunctorType& functor,
                        const std::string& label, uint64_t& kpID) {
   if (Kokkos::Tools::profileLibraryLoaded()) {
     Kokkos::Tools::endParallelScan(kpID);
@@ -415,8 +543,10 @@ void end_parallel_scan(ExecPolicy& policy, FunctorType& functor,
 }
 
 template <class ReducerType, class ExecPolicy, class FunctorType>
-void begin_parallel_reduce(ExecPolicy& policy, FunctorType& functor,
+auto begin_parallel_reduce(const ExecPolicy& policy, FunctorType& functor,
                            const std::string& label, uint64_t& kpID) {
+  using response_type = ToolResponse<ExecPolicy, FunctorType>;
+  response_type response{policy};
   if (Kokkos::Tools::profileLibraryLoaded()) {
     Kokkos::Impl::ParallelConstructName<FunctorType,
                                         typename ExecPolicy::work_tag>
@@ -426,16 +556,17 @@ void begin_parallel_reduce(ExecPolicy& policy, FunctorType& functor,
         &kpID);
   }
 #ifdef KOKKOS_ENABLE_TUNING
-  size_t context_id = Kokkos::Tools::Experimental::get_new_context_id();
-  Experimental::Impl::ReductionSwitcher<ReducerType>::tune(
-      context_id, label, policy, functor, Kokkos::ParallelReduceTag{});
+  size_t context_id = Kokkos::Tools::Experimental::get_current_context_id();
+  return response_type{Experimental::Impl::ReductionSwitcher<ReducerType>::tune(
+      context_id, label, policy, functor, Kokkos::ParallelReduceTag{})};
 #else
   (void)functor;
 #endif
+  return response;
 }
 
 template <class ReducerType, class ExecPolicy, class FunctorType>
-void end_parallel_reduce(ExecPolicy& policy, FunctorType& functor,
+void end_parallel_reduce(const ExecPolicy& policy, FunctorType& functor,
                          const std::string& label, uint64_t& kpID) {
   if (Kokkos::Tools::profileLibraryLoaded()) {
     Kokkos::Tools::endParallelReduce(kpID);
@@ -453,7 +584,7 @@ void end_parallel_reduce(ExecPolicy& policy, FunctorType& functor,
 #endif
 }
 
-}  // end namespace Impl
+}  // namespace Impl
 
 }  // namespace Tools
 
