@@ -109,9 +109,11 @@ TEST(defaultdevicetype, shared_space) {
     GTEST_SKIP()
         << "skipping because specified arch does not support page migration";
 #endif
-#if defined(KOKKOS_ENABLE_SYCL) && !defined(KOKKOS_ARCH_INTEL_GPU)
-  GTEST_SKIP()
-      << "skipping because clock_tic is only defined for sycl+intel gpu";
+#if defined(KOKKOS_ENABLE_SYCL) &&      \
+    (!defined(KOKKOS_ARCH_INTEL_GPU) || \
+     !defined(KOKKOS_ENABLE_SYCL_RELOCATABLE_DEVICE_CODE))
+  GTEST_SKIP() << "skipping because clock_tic is only defined for sycl+intel "
+                  "gpu and with rdc support";
 #endif
 
   const unsigned int numRepetitions      = 10;
