@@ -19,10 +19,6 @@
 
 namespace {
 
-// Extended lambdas in parallel_for and parallel_reduce will not compile if
-// KOKKOS_ENABLE_CUDA_LAMBDA is off
-#if !defined(KOKKOS_ENABLE_CUDA) || defined(KOKKOS_ENABLE_CUDA_LAMBDA)
-
 struct TeamTeamCombinedReducer {
  public:
   void test_team_thread_range_only_scalars(const int n) {
@@ -510,6 +506,4 @@ TEST(TEST_CATEGORY, team_vector_range_combined_reducers) {
   tester.test_team_vector_range_combined_reducers(9);
 }
 
-#endif
-
 }  // namespace
