@@ -170,7 +170,7 @@ TEST(TEST_CATEGORY_DEATH, view_memory_access_violations_from_device) {
   }
 
 #if defined(KOKKOS_ENABLE_SYCL) && defined(NDEBUG)  // FIXME_SYCL
-  if (std::is_same<ExecutionSpace, Kokkos::Experimental::SYCL>::value) {
+  if (std::is_same<ExecutionSpace, Kokkos::SYCL>::value) {
     GTEST_SKIP() << "skipping SYCL device-side abort does not work when NDEBUG "
                     "is defined";
   }
