@@ -1,46 +1,18 @@
-/*
 //@HEADER
 // ************************************************************************
 //
-//                        Kokkos v. 3.0
-//       Copyright (2020) National Technology & Engineering
+//                        Kokkos v. 4.0
+//       Copyright (2022) National Technology & Engineering
 //               Solutions of Sandia, LLC (NTESS).
 //
 // Under the terms of Contract DE-NA0003525 with NTESS,
 // the U.S. Government retains certain rights in this software.
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
+// Part of Kokkos, under the Apache License v2.0 with LLVM Exceptions.
+// See https://kokkos.org/LICENSE for license information.
+// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 //
-// 1. Redistributions of source code must retain the above copyright
-// notice, this list of conditions and the following disclaimer.
-//
-// 2. Redistributions in binary form must reproduce the above copyright
-// notice, this list of conditions and the following disclaimer in the
-// documentation and/or other materials provided with the distribution.
-//
-// 3. Neither the name of the Corporation nor the names of the
-// contributors may be used to endorse or promote products derived from
-// this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY NTESS "AS IS" AND ANY
-// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NTESS OR THE
-// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
-// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
-// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-//
-// Questions? Contact Christian R. Trott (crtrott@sandia.gov)
-//
-// ************************************************************************
 //@HEADER
-*/
 /**
  * Before digging in to the code, it's worth taking a moment to review this
  * design. Fundamentally, what we're looking to do is allow people to test that
@@ -92,26 +64,25 @@ struct EventBase;  // forward declaration
 using EventBasePtr = std::shared_ptr<EventBase>;
 using event_vector = std::vector<EventBasePtr>;
 
-/**
- * @brief Base case of a recursive reduction using templates
- * Should be replaced with a fold in C++17
- */
+// unique identifier for derived event classes
+template <class EventDerived,
+          std::enable_if_t<std::is_base_of_v<EventBase, EventDerived> &&
+                           std::is_final_v<EventDerived>>* = nullptr>
+uintptr_t event_type_uid() {
+  static char x{};
+  return reinterpret_cast<uintptr_t>(&x);
+}
 
-inline bool are_valid() { return true; }
-
-/**
- * @brief Recursive reduction to check whether any pointer in a set is null
- *
- * @tparam Head Type of the pointer to examine
- * @tparam Tail Types of the rest of the pointers
- * @param head The pointer to examine
- * @param tail The rest of the pointers
- * @return true if no pointer is null, false otherwise
- *
- */
-template <class Head, class... Tail>
-bool are_valid(const Head& head, const Tail&... tail) {
-  return (head != nullptr) && (are_valid(tail...));
+// runtime check to determine if an event object of EventBase class is in fact
+// of a particular EventDerived class in which case it is safe to downcast
+// (essentially some hand-rolled form of dynamic pointer casting w/o using RTTI)
+template <
+    class EventDerived, class SomeEventBasePtr,
+    std::enable_if_t<std::is_base_of_v<EventBase, EventDerived> &&
+                     std::is_final_v<EventDerived> &&
+                     std::is_same_v<SomeEventBasePtr, EventBasePtr>>* = nullptr>
+bool is_a(SomeEventBasePtr const& e) {
+  return e->kind() == event_type_uid<EventDerived>();
 }
 
 /**
@@ -153,10 +124,10 @@ struct function_traits<R (*)(A...)> {
   constexpr static int num_arguments = sizeof...(A);
   template <class Call, class... Args>
   static auto invoke_as(const Call& call, Args&&... args) {
-    if (!are_valid(std::dynamic_pointer_cast<A>(std::forward<Args>(args))...)) {
+    if (!(is_a<A>(std::forward<Args>(args)) && ...)) {
       return MatchDiagnostic{false, {"Types didn't match on arguments"}};
     }
-    return call(*std::dynamic_pointer_cast<A>(std::forward<Args>(args))...);
+    return call(*std::static_pointer_cast<A>(std::forward<Args>(args))...);
   }
 };
 
@@ -177,10 +148,10 @@ struct function_traits<R (C::*)(A...)> {
   constexpr static int num_arguments = sizeof...(A);
   template <class Call, class... Args>
   static auto invoke_as(const Call& call, Args&&... args) {
-    if (!are_valid(std::dynamic_pointer_cast<A>(std::forward<Args>(args))...)) {
+    if (!(is_a<A>(std::forward<Args>(args)) && ...)) {
       return MatchDiagnostic{false, {"Types didn't match on arguments"}};
     }
-    return call(*std::dynamic_pointer_cast<A>(std::forward<Args>(args))...);
+    return call(*std::static_pointer_cast<A>(std::forward<Args>(args))...);
   }
 };
 
@@ -202,10 +173,10 @@ struct function_traits<R (C::*)(A...) const>  // const
   constexpr static int num_arguments = sizeof...(A);
   template <class Call, class... Args>
   static auto invoke_as(const Call& call, Args&&... args) {
-    if (!are_valid(std::dynamic_pointer_cast<A>(std::forward<Args>(args))...)) {
+    if (!(is_a<A>(std::forward<Args>(args)) && ...)) {
       return MatchDiagnostic{false, {"Types didn't match on arguments"}};
     }
-    return call(*std::dynamic_pointer_cast<A>(std::forward<Args>(args))...);
+    return call(*std::static_pointer_cast<A>(std::forward<Args>(args))...);
   }
 };
 
@@ -218,7 +189,7 @@ struct function_traits<R (C::*)(A...) const>  // const
  * @tparam T The functor type
  */
 template <typename T>
-struct function_traits<T, Kokkos::Impl::void_t<decltype(&T::operator())> >
+struct function_traits<T, std::void_t<decltype(&T::operator())>>
     : public function_traits<decltype(&T::operator())> {};
 
 /**
@@ -334,6 +305,12 @@ struct EventBase {
   using PtrHandle                        = const void* const;
   virtual ~EventBase()                   = default;
   virtual std::string descriptor() const = 0;
+  virtual uintptr_t kind() const         = 0;
+};
+
+template <class Derived>
+struct UniquelyIdentifiableEventType : public EventBase {
+  uintptr_t kind() const override { return event_type_uid<Derived>(); }
 };
 
 /**
@@ -343,7 +320,7 @@ struct EventBase {
  * @tparam Derived CRTP, intended for use with dynamic_casts
  */
 template <class Derived>
-struct BeginOperation : public EventBase {
+struct BeginOperation : public UniquelyIdentifiableEventType<Derived> {
   const std::string name;
   const uint32_t deviceID;
   uint64_t kID;
@@ -367,7 +344,7 @@ struct BeginOperation : public EventBase {
  * same type
  */
 template <class Derived>
-struct EndOperation : public EventBase {
+struct EndOperation : public UniquelyIdentifiableEventType<Derived> {
   uint64_t kID;
   EndOperation(uint64_t k) : kID(k) {}
 
@@ -386,7 +363,8 @@ struct EndOperation : public EventBase {
  * type. So the different type names here are meaningful, even though the
  * classes are empty
  */
-struct BeginParallelForEvent : public BeginOperation<BeginParallelForEvent> {
+struct BeginParallelForEvent final
+    : public BeginOperation<BeginParallelForEvent> {
   static const std::string& begin_op_name() {
     static std::string value = "BeginParallelFor";
     return value;
@@ -394,7 +372,7 @@ struct BeginParallelForEvent : public BeginOperation<BeginParallelForEvent> {
   BeginParallelForEvent(std::string n, const uint32_t devID, uint64_t k)
       : BeginOperation<BeginParallelForEvent>(n, devID, k) {}
 };
-struct BeginParallelReduceEvent
+struct BeginParallelReduceEvent final
     : public BeginOperation<BeginParallelReduceEvent> {
   static const std::string& begin_op_name() {
     static std::string value = "BeginParallelReduce";
@@ -404,7 +382,8 @@ struct BeginParallelReduceEvent
   BeginParallelReduceEvent(std::string n, const uint32_t devID, uint64_t k)
       : BeginOperation<BeginParallelReduceEvent>(n, devID, k) {}
 };
-struct BeginParallelScanEvent : public BeginOperation<BeginParallelScanEvent> {
+struct BeginParallelScanEvent final
+    : public BeginOperation<BeginParallelScanEvent> {
   static const std::string& begin_op_name() {
     static std::string value = "BeginParallelScan";
     return value;
@@ -413,7 +392,7 @@ struct BeginParallelScanEvent : public BeginOperation<BeginParallelScanEvent> {
   BeginParallelScanEvent(std::string n, const uint32_t devID, uint64_t k)
       : BeginOperation<BeginParallelScanEvent>(n, devID, k) {}
 };
-struct BeginFenceEvent : public BeginOperation<BeginFenceEvent> {
+struct BeginFenceEvent final : public BeginOperation<BeginFenceEvent> {
   static const std::string& begin_op_name() {
     static std::string value = "BeginFence";
     return value;
@@ -423,7 +402,7 @@ struct BeginFenceEvent : public BeginOperation<BeginFenceEvent> {
       : BeginOperation<BeginFenceEvent>(n, devID, k) {}
 };
 
-struct EndParallelForEvent : public EndOperation<EndParallelForEvent> {
+struct EndParallelForEvent final : public EndOperation<EndParallelForEvent> {
   static const std::string& end_op_name() {
     static std::string value = "EndParallelFor";
     return value;
@@ -431,7 +410,8 @@ struct EndParallelForEvent : public EndOperation<EndParallelForEvent> {
 
   EndParallelForEvent(uint64_t k) : EndOperation<EndParallelForEvent>(k) {}
 };
-struct EndParallelReduceEvent : public EndOperation<EndParallelReduceEvent> {
+struct EndParallelReduceEvent final
+    : public EndOperation<EndParallelReduceEvent> {
   static const std::string& end_op_name() {
     static std::string value = "EndParallelReduce";
     return value;
@@ -440,7 +420,7 @@ struct EndParallelReduceEvent : public EndOperation<EndParallelReduceEvent> {
   EndParallelReduceEvent(uint64_t k)
       : EndOperation<EndParallelReduceEvent>(k) {}
 };
-struct EndParallelScanEvent : public EndOperation<EndParallelScanEvent> {
+struct EndParallelScanEvent final : public EndOperation<EndParallelScanEvent> {
   static const std::string& end_op_name() {
     static std::string value = "EndParallelScan";
     return value;
@@ -448,7 +428,7 @@ struct EndParallelScanEvent : public EndOperation<EndParallelScanEvent> {
 
   EndParallelScanEvent(uint64_t k) : EndOperation<EndParallelScanEvent>(k) {}
 };
-struct EndFenceEvent : public EndOperation<EndFenceEvent> {
+struct EndFenceEvent final : public EndOperation<EndFenceEvent> {
   static const std::string& end_op_name() {
     static std::string value = "EndFence";
     return value;
@@ -457,7 +437,7 @@ struct EndFenceEvent : public EndOperation<EndFenceEvent> {
   EndFenceEvent(uint64_t k) : EndOperation<EndFenceEvent>(k) {}
 };
 
-struct InitEvent : public EventBase {
+struct InitEvent final : public UniquelyIdentifiableEventType<InitEvent> {
   int load_sequence;
   uint64_t version_number;
   uint32_t num_device_infos;
@@ -475,11 +455,13 @@ struct InitEvent : public EventBase {
         num_device_infos(n_d_i),
         device_infos(d_i) {}
 };
-struct FinalizeEvent : public EventBase {
+struct FinalizeEvent final
+    : public UniquelyIdentifiableEventType<FinalizeEvent> {
   std::string descriptor() const override { return "FinalizeEvent{}"; }
 };
 
-struct ParseArgsEvent : public EventBase {
+struct ParseArgsEvent final
+    : public UniquelyIdentifiableEventType<ParseArgsEvent> {
   int num_args;
   char** args;
 
@@ -494,26 +476,29 @@ struct ParseArgsEvent : public EventBase {
   }
   ParseArgsEvent(int n_a, char** a) : num_args(n_a), args(a) {}
 };
-struct PrintHelpEvent : public EventBase {
+struct PrintHelpEvent final
+    : public UniquelyIdentifiableEventType<PrintHelpEvent> {
   char* prog_name;
   std::string descriptor() const override {
     return "PrintHelpEvent { Program Name: \"" + std::string(prog_name) + "\"}";
   }
   PrintHelpEvent(char* p_n) : prog_name(p_n) {}
 };
-struct PushRegionEvent : public EventBase {
+struct PushRegionEvent final
+    : public UniquelyIdentifiableEventType<PushRegionEvent> {
   std::string name;
   std::string descriptor() const override {
     return "PushRegionEvent { Region Name: \"" + name + "\" }";
   }
   PushRegionEvent(std::string n) : name(n) {}
 };
-struct PopRegionEvent : public EventBase {
+struct PopRegionEvent final
+    : public UniquelyIdentifiableEventType<PopRegionEvent> {
   std::string descriptor() const override { return "PopRegionEvent{}"; }
 };
 
 template <class Derived>
-struct DataEvent : public EventBase {
+struct DataEvent : public UniquelyIdentifiableEventType<Derived> {
   using SpaceHandleType = Kokkos::Profiling::SpaceHandle;
   SpaceHandleType handle;
   std::string name;
@@ -532,20 +517,21 @@ struct DataEvent : public EventBase {
       : handle(h), name(n), ptr(p), size(s) {}
 };
 
-struct AllocateDataEvent : public DataEvent<AllocateDataEvent> {
+struct AllocateDataEvent final : public DataEvent<AllocateDataEvent> {
   static std::string event_name() { return "AllocateDataEvent"; }
   AllocateDataEvent(DataEvent::SpaceHandleType h, std::string n,
                     EventBase::PtrHandle p, uint64_t s)
       : DataEvent<AllocateDataEvent>(h, n, p, s) {}
 };
-struct DeallocateDataEvent : public DataEvent<DeallocateDataEvent> {
+struct DeallocateDataEvent final : public DataEvent<DeallocateDataEvent> {
   static std::string event_name() { return "DeallocateDataEvent"; }
   DeallocateDataEvent(DataEvent::SpaceHandleType h, std::string n,
                       EventBase::PtrHandle p, uint64_t s)
       : DataEvent<DeallocateDataEvent>(h, n, p, s) {}
 };
 
-struct CreateProfileSectionEvent : public EventBase {
+struct CreateProfileSectionEvent final
+    : public UniquelyIdentifiableEventType<CreateProfileSectionEvent> {
   std::string name;
   uint32_t id;
   std::string descriptor() const override {
@@ -556,7 +542,8 @@ struct CreateProfileSectionEvent : public EventBase {
 };
 
 template <class Derived>
-struct ProfileSectionManipulationEvent : public EventBase {
+struct ProfileSectionManipulationEvent
+    : public UniquelyIdentifiableEventType<Derived> {
   uint32_t id;
   std::string descriptor() const override {
     std::stringstream s;
@@ -566,26 +553,26 @@ struct ProfileSectionManipulationEvent : public EventBase {
   ProfileSectionManipulationEvent(uint32_t d_i) : id(d_i){};
 };
 
-struct StartProfileSectionEvent
+struct StartProfileSectionEvent final
     : public ProfileSectionManipulationEvent<StartProfileSectionEvent> {
   static std::string event_name() { return "StartProfileSectionEvent"; }
   StartProfileSectionEvent(uint32_t d_i)
       : ProfileSectionManipulationEvent<StartProfileSectionEvent>(d_i){};
 };
-struct StopProfileSectionEvent
+struct StopProfileSectionEvent final
     : public ProfileSectionManipulationEvent<StopProfileSectionEvent> {
   static std::string event_name() { return "StopProfileSectionEvent"; }
   StopProfileSectionEvent(uint32_t d_i)
       : ProfileSectionManipulationEvent<StopProfileSectionEvent>(d_i){};
 };
-struct DestroyProfileSectionEvent
+struct DestroyProfileSectionEvent final
     : public ProfileSectionManipulationEvent<DestroyProfileSectionEvent> {
   static std::string event_name() { return "DestroyProfileSectionEvent"; }
   DestroyProfileSectionEvent(uint32_t d_i)
       : ProfileSectionManipulationEvent<DestroyProfileSectionEvent>(d_i){};
 };
 
-struct ProfileEvent : public EventBase {
+struct ProfileEvent final : public UniquelyIdentifiableEventType<ProfileEvent> {
   std::string name;
   std::string descriptor() const override {
     return "ProfileEvent {\"" + name + "\"}";
@@ -593,7 +580,8 @@ struct ProfileEvent : public EventBase {
   ProfileEvent(std::string n) : name(n) {}
 };
 
-struct BeginDeepCopyEvent : public EventBase {
+struct BeginDeepCopyEvent final
+    : public UniquelyIdentifiableEventType<BeginDeepCopyEvent> {
   using SpaceHandleType = Kokkos::Profiling::SpaceHandle;
   SpaceHandleType src_handle;
   std::string src_name;
@@ -623,12 +611,13 @@ struct BeginDeepCopyEvent : public EventBase {
         dst_ptr(d_p),
         size(s) {}
 };
-struct EndDeepCopyEvent : public EventBase {
+struct EndDeepCopyEvent final
+    : public UniquelyIdentifiableEventType<EndDeepCopyEvent> {
   std::string descriptor() const override { return "EndDeepCopyEvent{}"; }
 };
 
 template <class Derived>
-struct DualViewEvent : public EventBase {
+struct DualViewEvent : public UniquelyIdentifiableEventType<Derived> {
   std::string name;
   EventBase::PtrHandle ptr;
   bool is_device;
@@ -641,18 +630,19 @@ struct DualViewEvent : public EventBase {
     return s.str();
   }
 };
-struct DualViewModifyEvent : public DualViewEvent<DualViewModifyEvent> {
+struct DualViewModifyEvent final : public DualViewEvent<DualViewModifyEvent> {
   static std::string event_name() { return "DualViewModifyEvent"; }
   DualViewModifyEvent(std::string n, EventBase::PtrHandle p, bool i_d)
       : DualViewEvent(n, p, i_d) {}
 };
-struct DualViewSyncEvent : public DualViewEvent<DualViewSyncEvent> {
+struct DualViewSyncEvent final : public DualViewEvent<DualViewSyncEvent> {
   static std::string event_name() { return "DualViewSyncEvent"; }
   DualViewSyncEvent(std::string n, EventBase::PtrHandle p, bool i_d)
       : DualViewEvent(n, p, i_d) {}
 };
 
-struct DeclareMetadataEvent : public EventBase {
+struct DeclareMetadataEvent final
+    : public UniquelyIdentifiableEventType<DeclareMetadataEvent> {
   std::string key;
   std::string value;
   std::string descriptor() const override {
@@ -661,7 +651,9 @@ struct DeclareMetadataEvent : public EventBase {
   DeclareMetadataEvent(std::string k, std::string v) : key(k), value(v) {}
 };
 
-struct ProvideToolProgrammingInterfaceEvent : public EventBase {
+struct ProvideToolProgrammingInterfaceEvent final
+    : public UniquelyIdentifiableEventType<
+          ProvideToolProgrammingInterfaceEvent> {
   using Interface = Kokkos::Tools::Experimental::ToolProgrammingInterface;
 
   uint32_t num_functions;
@@ -673,7 +665,8 @@ struct ProvideToolProgrammingInterfaceEvent : public EventBase {
            std::to_string(num_functions) + "}";
   }
 };
-struct RequestToolSettingsEvent : public EventBase {
+struct RequestToolSettingsEvent final
+    : public UniquelyIdentifiableEventType<RequestToolSettingsEvent> {
   using Settings = Kokkos::Tools::Experimental::ToolSettings;
 
   uint32_t num_settings;
@@ -686,7 +679,7 @@ struct RequestToolSettingsEvent : public EventBase {
 };
 
 template <class Derived>
-struct TypeDeclarationEvent : public EventBase {
+struct TypeDeclarationEvent : public UniquelyIdentifiableEventType<Derived> {
   std::string name;
   size_t variable_id;
   Kokkos::Tools::Experimental::VariableInfo info;
@@ -698,14 +691,14 @@ struct TypeDeclarationEvent : public EventBase {
                        Kokkos::Tools::Experimental::VariableInfo i)
       : name(n), variable_id(v_i), info(i) {}
 };
-struct DeclareOutputTypeEvent
+struct DeclareOutputTypeEvent final
     : public TypeDeclarationEvent<DeclareOutputTypeEvent> {
   static std::string event_name() { return "DeclarateOutputTypeEvent"; }
   DeclareOutputTypeEvent(std::string n, size_t v_i,
                          Kokkos::Tools::Experimental::VariableInfo i)
       : TypeDeclarationEvent(n, v_i, i) {}
 };
-struct DeclareInputTypeEvent
+struct DeclareInputTypeEvent final
     : public TypeDeclarationEvent<DeclareInputTypeEvent> {
   static std::string event_name() { return "DeclareInputTypeEvent"; }
   DeclareInputTypeEvent(std::string n, size_t v_i,
@@ -713,7 +706,8 @@ struct DeclareInputTypeEvent
       : TypeDeclarationEvent(n, v_i, i) {}
 };
 
-struct RequestOutputValuesEvent : public EventBase {
+struct RequestOutputValuesEvent final
+    : public UniquelyIdentifiableEventType<RequestOutputValuesEvent> {
   size_t context;
   size_t num_inputs;
   std::vector<Kokkos::Tools::Experimental::VariableValue> inputs;
@@ -733,14 +727,16 @@ struct RequestOutputValuesEvent : public EventBase {
       : context(c), num_inputs(n_i), inputs(i), num_outputs(n_o), outputs(o) {}
 };
 
-struct BeginContextEvent : public EventBase {
+struct BeginContextEvent final
+    : public UniquelyIdentifiableEventType<BeginContextEvent> {
   size_t context;
   std::string descriptor() const override {
     return "ContextBeginEvent{ " + std::to_string(context) + "}";
   }
   BeginContextEvent(size_t c) : context(c) {}
 };
-struct EndContextEvent : public EventBase {
+struct EndContextEvent final
+    : public UniquelyIdentifiableEventType<EndContextEvent> {
   size_t context;
   Kokkos::Tools::Experimental::VariableValue value;
   std::string descriptor() const override {
@@ -750,7 +746,8 @@ struct EndContextEvent : public EventBase {
       : context(c), value(v) {}
 };
 
-struct OptimizationGoalDeclarationEvent : public EventBase {
+struct OptimizationGoalDeclarationEvent final
+    : public UniquelyIdentifiableEventType<OptimizationGoalDeclarationEvent> {
   size_t context;
   Kokkos::Tools::Experimental::OptimizationGoal goal;
   std::string descriptor() const override {
@@ -975,7 +972,8 @@ static uint64_t last_kernel_id;
 static uint32_t last_section_id;
 
 /** Subscribes to all of the requested callbacks */
-static void set_tool_events_impl(const ToolValidatorConfiguration& config) {
+static inline void set_tool_events_impl(
+    const ToolValidatorConfiguration& config) {
   Kokkos::Tools::Experimental::pause_tools();  // remove all events
   if (config.profiling.kernels) {
     Kokkos::Tools::Experimental::set_begin_parallel_for_callback(
@@ -1195,8 +1193,7 @@ template <int priority, class Config, class... Configs>
 void listen_tool_events_impl(std::integral_constant<int, priority> prio,
                              ToolValidatorConfiguration& in, Config conf,
                              Configs... configs) {
-  invoke_config(in, conf,
-                std::integral_constant<bool, priority == conf.value>{});
+  invoke_config(in, conf, std::bool_constant<priority == conf.value>{});
   listen_tool_events_impl(prio, in, configs...);
 }
 template <class... Configs>
